/*
 * flash_srec.c
 *
 *  Created on: Mar 21, 2017
 *      Author: gjacobso01
 */
#include "hal_data.h"
#include "common_utils.h"
#include "flash_srec.h"

extern const flash_instance_t g_flash0;

#define FLASH_CODE_AREA_START   (0x20000)      // Application area starting from Block 11
#define FLASH_CODE_AREA_END     (0x80000)
#define FLASH_BLOCK_SIZE        (0x8000)        // 32k
#define FLASH_WRITE_SIZE        (128U)
#define FLASH_WRITE_MASK        (FLASH_WRITE_SIZE-1)
#define RAM_AREA_START          (0x1FFE0000)
#define OSIS_CONFIG_AREA        (0x0100A150)    /* OCD/Serial Programmer ID Setting Register (OSIS) */

static flash_write_cache_t fcache_info;
//static FIL                 SrecFile;
/*
 * HexChecksum
 *
 * This function verifies the checksum of an Srec format file record starting with the first
 * character after the "Sn" at the start of the record.
 *
 * Input: Pointer to record to verify
 * Output: returns TRUE if checksum is correct.
 *
 */
static bool HexChecksum(char *hexstr)
{
    char * p;
    char * q;
    uint32_t chksum;
    char hex[3];

    chksum = 1;

    for (p = strtok(hexstr, "\n\r"); *p != 0; p+=2)
    {
        chksum += (uint32_t)strtol(strncpy(hex,p,2), &q ,16);
    }
    chksum &= 0xFF;
    return((uint8_t)chksum == 0);
}

static uint32_t StartFlashProgramming(void)
{
    fsp_err_t status;
    flash_info_t flash_info;
    flash_result_t result;
    uint32_t address;

#if 0
    status = g_flash0.p_api->open(g_flash0.p_ctrl, g_flash0.p_cfg);
    if (status) __BKPT();
#endif
    status = g_flash0.p_api->infoGet(g_flash0.p_ctrl, &flash_info);
    if (status) __BKPT();

    fcache_info.dirty = false;
    fcache_info.start_address = FLASH_CODE_AREA_START;
    fcache_info.next_address   = FLASH_CODE_AREA_START + FLASH_WRITE_SIZE;
    memset(fcache_info.write_block, 0xFF, sizeof(fcache_info.write_block));

    for (address = FLASH_CODE_AREA_START; address < FLASH_CODE_AREA_END; address += FLASH_BLOCK_SIZE)
    {
        status = g_flash0.p_api->blankCheck(g_flash0.p_ctrl, address, FLASH_BLOCK_SIZE, &result);
        APP_PRINT("__BKPT %d address:%x\n", status, address);
        if  (status) __BKPT();
        APP_PRINT("check status %d check address: %X end address: %x \n", status, address, FLASH_CODE_AREA_END);
        if (FLASH_RESULT_NOT_BLANK == result)
        {
            APP_PRINT("start disable_irq \n ");
           __disable_irq();
           APP_PRINT("start status = g_flash0 \n");
           status = g_flash0.p_api->erase(g_flash0.p_ctrl, address, 1);
           APP_PRINT(" status %d \n ", status);
           if  (status) __BKPT();
           APP_PRINT("end if (status) __BKPT() \n ");
           __enable_irq();
        }
    }

    return(0);
}

static uint32_t EndFlashProgramming(void)
{
    fsp_err_t status = 0;

    if (fcache_info.dirty)        // Flush the current buffer
    {
        __disable_irq();
        status = g_flash0.p_api->write(g_flash0.p_ctrl, (uint32_t)fcache_info.write_block, fcache_info.start_address, FLASH_WRITE_SIZE);
        __enable_irq();
        if (status) __BKPT();
    }

#if 0
    status = g_flash0.p_api->close(g_flash0.p_ctrl);
#endif
    return(status);
}

static void write_flash(uint32_t addr, uint8_t * pData, uint32_t len)
{
    fsp_err_t status;
    uint32_t i;
    uint8_t idx;

    if (addr < fcache_info.start_address) __BKPT();     // We can't re-program flash without erasing it!!!

    /*
     *  The address we are trying to program is outside the range of the current write line
     */
    if (addr >= fcache_info.next_address)
    {
        if (fcache_info.dirty)        // Flush the current buffer
        {
            __disable_irq();
            status = g_flash0.p_api->write(g_flash0.p_ctrl, (uint32_t)fcache_info.write_block, fcache_info.start_address, FLASH_WRITE_SIZE);
            __enable_irq();
            if (status) __BKPT();
        }
        memset(fcache_info.write_block, 0xFF, FLASH_WRITE_SIZE);
        fcache_info.start_address = addr & (~FLASH_WRITE_MASK);
        fcache_info.next_address = fcache_info.start_address + FLASH_WRITE_SIZE;
    }

    /*
     *  The address is within the current write line
     */
    for (i = 0, idx = (addr & FLASH_WRITE_MASK); i<len; i++, idx++)
    {
        /*
         *  If the address is outside the write line flush the buffer first then fill the buffer
         */
        if ((i+addr) == fcache_info.next_address)
        {
            __disable_irq();
            status = g_flash0.p_api->write(g_flash0.p_ctrl, (uint32_t) fcache_info.write_block, fcache_info.start_address, FLASH_WRITE_SIZE);
            __enable_irq();
            if (status) __BKPT();

            memset(fcache_info.write_block, 0xFF, FLASH_WRITE_SIZE);
            fcache_info.start_address = fcache_info.next_address;
            fcache_info.next_address =  fcache_info.start_address + FLASH_WRITE_SIZE;
            idx = 0;
        }
        /*
         *  The address is within the write line, fill the write buffer
         */
        fcache_info.write_block[idx] = pData[i];
     }
     fcache_info.dirty = true;
}

/*
 * ParseSrecFile
 *
 * This function populates the memory region with the binary image in the Intel Hex file.
 *
 * Input:
 *          File pointer to Intel hex file, file must be open before calling this function
 *          Low Address value - beginning of the address range
 *          High Address value - end of the address range
 *          Pointer to memory image to populate.
 *
 * Output:
 *          Memory image is updated with binary image from the Intel hex file.
 *
 */
//uint32_t ParseSrecFile(char * filename)
//{
//    FRESULT     rst;
//    char        buffer[FF_MAX_LFN];
//    char        info[FF_MAX_LFN/2];
//    char        rec[2];
//    char        hex[3]={0};
//    char        addr[10]={0};
//    char        *p;
//    char        *q;
//
//    FSP_PARAMETER_NOT_USED(info);
//
//    uint32_t Address;
//    uint32_t ByteCount;
//    uint8_t data[16];
//
//    rst = f_open( &SrecFile, filename, FA_OPEN_EXISTING | FA_READ);
//    if (FR_OK != rst)
//    {
//        APP_PRINT("Error (%s)\n", convert_FRESULT_to_str( rst));
//        return (1);
//    }
//
//    rst = f_lseek( &SrecFile, 0);
//    if (FR_OK != rst)
//    {
//        APP_PRINT("Error (%s)\n", convert_FRESULT_to_str( rst));
//        f_close(&SrecFile);
//        return (1);
//    }
//
//    do {
//        f_gets(buffer, sizeof(buffer), &SrecFile);
////        strncpy(rec, buffer,2);
//        memcpy(rec, buffer, sizeof(rec));
//
//        ByteCount = (uint32_t)strtol(strncpy(hex,buffer+2,2), &p ,16);
//        p = buffer+4;
//
//        if ((rec[0] != 'S') || !HexChecksum(buffer+2))
//        {
//            f_close(&SrecFile);
//
//            return (1);
//        }
//
//        switch (rec[1]) {
//            case '0' :  // Copy the Header information
//                        p = buffer+8;
//                        for (uint32_t i=0; i<ByteCount-4; i++)
//                        {
//                            info[i] = (char)strtol(strncpy(hex, p+(i*2),2), &q, 16);
//                        }
//                        StartFlashProgramming();
//                        break;
//            case '1' :  // 16 Bit address Record
//                        strncpy(addr,p,4);
//                        Address = (uint32_t)strtol(addr,&p, 16);  // Get the address of this line of data
//                        /*
//                         * Range check for valid area - we only support address range outside of the
//                         * bootloader its self.
//                         */
//                       if ((Address < FLASH_CODE_AREA_START) || (Address >= FLASH_CODE_AREA_END))
//                        {
//                           if (Address >= RAM_AREA_START) break;
//                            EndFlashProgramming();
//                            //f_close(&SrecFile);
//                            return (2);
//                        }
//
//                        /*
//                         * Calculate the actual byte count from the s-record count field
//                         * Remove address (3 bytes) & checksum (1 bytes) to get the number of data bytes
//                         */
//                        ByteCount = (ByteCount - 3);
//                        /*
//                         * Set pointer to the first character of the data field
//                         * start of data field = record type(2) + count(2) + address(6)
//                         */
//                        p = buffer+8;
//                        for (uint32_t i=0; i<ByteCount; i++)
//                        {
//                           data[i] = (uint8_t)strtol(strncpy(hex, p+(i*2),2), &q, 16);
//                        }
//                        write_flash(Address, data, ByteCount);
//                        break;
//            case '2' :  // 24 Bit Address Record
//                        strncpy(addr,p,6);
//                        Address = (uint32_t)strtol(addr,&p, 16);  // Get the address of this line of data
//                        /*
//                         * Range check for valid area - we only support address range outside of the
//                         * bootloader its self.
//                         */
//                       if ((Address < FLASH_CODE_AREA_START) || (Address >= FLASH_CODE_AREA_END))
//                        {
//                           if (Address >= RAM_AREA_START) break;
//                            EndFlashProgramming();
//                            f_close(&SrecFile);
//                            return (2);
//                        }
//
//                        /*
//                         * Calculate the actual byte count from the s-record count field
//                         * Remove address (3 bytes) & checksum (1 bytes) to get the number of data bytes
//                         */
//                        ByteCount = (ByteCount - 4);
//                        /*
//                         * Set pointer to the first character of the data field
//                         * start of data field = record type(2) + count(2) + address(6)
//                         */
//                        p = buffer+10;
//                        for (uint32_t i=0; i<ByteCount; i++)
//                        {
//                           data[i] = (uint8_t)strtol(strncpy(hex, p+(i*2),2), &q, 16);
//                        }
//                        write_flash(Address, data, ByteCount);
//                        break;
//            case '3' :  // 32 Bit Address Record
//                        strncpy(addr,p,8);
//                        Address = (uint32_t)strtol(addr,&p, 16);  // Get the address of this line of data
//                        /*
//                         * Range check for valid area - we only support address range outside of the
//                         * bootloader its self.
//                         */
//                        if ((Address < FLASH_CODE_AREA_START) || (Address >= FLASH_CODE_AREA_END))
//                        {
//                            if ((Address >= RAM_AREA_START)||(Address == OSIS_CONFIG_AREA)) break;
//                            EndFlashProgramming();
//                            f_close(&SrecFile);
//                            return (2);
//                        }
//                        /*
//                         * Calculate the actual byte count from the s-record count field
//                         * Remove address (4 bytes) & checksum (1 bytes) to get the number of data bytes
//                         */
//                        ByteCount = (ByteCount - 5);
//                        /*
//                         * Set pointer to the first character of the data field
//                         * start of data field = record type(2) + count(2) + address(8)
//                         */
//                        p = buffer+12;
//                        for (uint32_t i=0; i<ByteCount; i++)
//                        {
//                           data[i] = (uint8_t)strtol(strncpy(hex, p+(i*2),2), &q, 16);
//                        }
//                        write_flash(Address, data, ByteCount);
//                        break;
//            case '5' :  // Ignore the count record(s)
//            case '6' :
//                        break;
//            case '7' :  // Start Address Record (Termination of the s-rec file)
//            case '8' :
//            case '9' :
//                        EndFlashProgramming();
//                        break;
//            default :
//                f_close(&SrecFile);
//                return (1);
//        }
//    } while (!f_eof(&SrecFile));
//
//    f_close(&SrecFile);
//    return(0);
//}


#define NUM_STRING_DESCRIPTOR          (7U)
#define LINE_CODING_LENGTH          (0x07U)
#define READ_BUF_SIZE                   64

/* Pointers to store USB descriptors */
extern uint8_t g_apl_device[];
extern uint8_t g_apl_configuration[];
extern uint8_t g_apl_hs_configuration[];
extern uint8_t g_apl_qualifier_descriptor[];
extern uint8_t * g_apl_string_table[];

const usb_descriptor_t g_usb_pcdc_descriptor =
{
 g_apl_device,                   /* Pointer to the device descriptor */
 g_apl_configuration,            /* Pointer to the configuration descriptor for Full-speed */
 g_apl_hs_configuration,         /* Pointer to the configuration descriptor for Hi-speed */
 g_apl_qualifier_descriptor,     /* Pointer to the qualifier descriptor */
 g_apl_string_table,             /* Pointer to the string descriptor table */
 NUM_STRING_DESCRIPTOR           /* Num entry String Descriptor */
};


usb_status_t            usb_event;
static bool  b_usb_attach = false;
uint8_t g_buf[READ_BUF_SIZE]            = {0};
uint32_t update_timerout = 400000;

uint8_t get_usb_data(uint32_t get_line, uint8_t *data_p)
{
    uint32_t get_timerout = 500000; //获取超时检测
    uint8_t get_line_str[10] = {0};
    fsp_err_t err                           = FSP_SUCCESS;
    usb_event_info_t    event_info          = {0};

    static usb_pcdc_linecoding_t g_line_coding;

    sprintf((char *)get_line_str, "%d", get_line);  //获取第几行

    R_USB_Write(&g_basic0_ctrl, get_line_str, strlen((char *)get_line_str), USB_CLASS_PCDC);

    while (get_timerout)
    {
        if(b_usb_attach == false)   //如果没检测到USB则进入主程序
        {
            if(update_timerout > 0)
                update_timerout--;
        }
        get_timerout--;
        /* Obtain USB related events */
        err = R_USB_EventGet (&event_info, &usb_event);

        /* Handle error */
        if (FSP_SUCCESS != err)
        {
            /* Turn ON RED LED to indicate fatal error */
            //TURN_RED_ON
            APP_ERR_TRAP(err);
        }

        /* USB event received by R_USB_EventGet */
        switch (usb_event)
        {
            case USB_STATUS_CONFIGURED:
            {
                err = R_USB_Read (&g_basic0_ctrl, g_buf, READ_BUF_SIZE, USB_CLASS_PCDC);
                /* Handle error */
                if (FSP_SUCCESS != err)
                {
                    /* Turn ON RED LED to indicate fatal error */
                    //TURN_RED_ON
                    APP_ERR_TRAP(err);
                }
                APP_PRINT("USB_STATUS_CONFIGURED\r\n");
                break;
            }

            case USB_STATUS_READ_COMPLETE:
            {
                //memset(g_buf, 0, READ_BUF_SIZE);
                if(b_usb_attach)
                {
                    err = R_USB_Read (&g_basic0_ctrl, g_buf, READ_BUF_SIZE, USB_CLASS_PCDC);
                }
                /* Handle error */
                if (FSP_SUCCESS != err)
                {
                    /* Turn ON RED LED to indicate fatal error */
                    //TURN_RED_ON
                    APP_ERR_TRAP(err);
                }
                memcpy(data_p, g_buf, READ_BUF_SIZE);
                APP_PRINT("usb rec:%s\r\n", g_buf);
                return 1;
                break;
            }

            case USB_STATUS_REQUEST : /* Receive Class Request */
            {
                /* Check for the specific CDC class request IDs */
                if (USB_PCDC_SET_LINE_CODING == (event_info.setup.request_type & USB_BREQUEST))
                {
                    err =  R_USB_PeriControlDataGet (&g_basic0_ctrl, (uint8_t *) &g_line_coding, LINE_CODING_LENGTH );
                    /* Handle error */
                    if (FSP_SUCCESS != err)
                    {
                        /* Turn ON RED LED to indicate fatal error */
                        //TURN_RED_ON
                        APP_ERR_TRAP(err);
                    }
                    APP_PRINT("USB_PCDC_SET_LINE_CODING\r\n");
                }
                else if (USB_PCDC_GET_LINE_CODING == (event_info.setup.request_type & USB_BREQUEST))
                {
                    err =  R_USB_PeriControlDataSet (&g_basic0_ctrl, (uint8_t *) &g_line_coding, LINE_CODING_LENGTH );
                    /* Handle error */
                    if (FSP_SUCCESS != err)
                    {
                        /* Turn ON RED LED to indicate fatal error */
                        //TURN_RED_ON
                        APP_ERR_TRAP(err);
                    }
                    APP_PRINT("USB_PCDC_GET_LINE_CODING\r\n");
                }
                else if (USB_PCDC_SET_CONTROL_LINE_STATE == (event_info.setup.request_type & USB_BREQUEST))
                {
                    err = R_USB_PeriControlStatusSet (&g_basic0_ctrl, USB_SETUP_STATUS_ACK);
                    /* Handle error */
                    if (FSP_SUCCESS != err)
                        //if (FSP_SUCCESS != g_err)
                    {
                        /* Turn ON RED LED to indicate fatal error */
                        //TURN_RED_ON
                        APP_ERR_TRAP(err);
                    }
                    APP_PRINT("USB_PCDC_SET_CONTROL_LINE_STATE\r\n");
                }
                else
                {
                    /* none */
                }

                break;
            }

            case USB_STATUS_DETACH:
            case USB_STATUS_SUSPEND:
            {
                APP_PRINT("USB_STATUS_DETACH or USB_STATUS_SUSPEND\r\n");
                b_usb_attach = false;
                memset (g_buf, 0, sizeof(g_buf));
                break;
            }
            case USB_STATUS_RESUME:
            {
                APP_PRINT("USB_STATUS_RESUME\r\n");
                b_usb_attach = true;
                break;
            }
            default:
            {
                break;
            }
        }
    }
    if(get_timerout == 0)
        return 0;
    else
        return 1;
}

/*
 * ParseSrecFile
 *
 * This function populates the memory region with the binary image in the Intel Hex file.
 *
 * Input:
 *          File pointer to Intel hex file, file must be open before calling this function
 *          Low Address value - beginning of the address range
 *          High Address value - end of the address range
 *          Pointer to memory image to populate.
 *
 * Output:
 *          Memory image is updated with binary image from the Intel hex file.
 *
 */
#define FF_MAX_LFN      255
uint32_t USBParseSrecFile(void)
{
    char        buffer[FF_MAX_LFN];
    char        info[FF_MAX_LFN/2];
    char        rec[2];
    char        hex[3]={0};
    char        addr[10]={0};
    char        *p;
    char        *q;

    FSP_PARAMETER_NOT_USED(info);

    uint32_t Address;
    uint32_t ByteCount;
    uint8_t data[16];
    uint32_t usb_get_line = 0;

    APP_PRINT("USBParseSrecFile\r\n");

    do {
        if(get_usb_data(usb_get_line, buffer) == 0)
            continue;
        usb_get_line++;
        //f_gets(buffer, sizeof(buffer), &SrecFile);//获取数据
//        strncpy(rec, buffer,2);
        memcpy(rec, buffer, sizeof(rec));

        ByteCount = (uint32_t)strtol(strncpy(hex,buffer+2,2), &p ,16);
        p = buffer+4;

        if ((rec[0] != 'S') || !HexChecksum(buffer+2))
        {
            if(rec[0] == 'E')
            {
                bsp_prv_software_delay_loop(2000);
                return 0;
            }
            //f_close(&SrecFile);
            return (1);
        }

        switch (rec[1]) {
            case '0' :  // Copy the Header information
//                        p = buffer+8;
//                        for (uint32_t i=0; i<ByteCount-4; i++)
//                        {
//                            info[i] = (char)strtol(strncpy(hex, p+(i*2),2), &q, 16);
//                        }
                        StartFlashProgramming();
                        break;
            case '1' :  // 16 Bit address Record
                        strncpy(addr,p,4);
                        Address = (uint32_t)strtol(addr,&p, 16);  // Get the address of this line of data
                        /*
                         * Range check for valid area - we only support address range outside of the
                         * bootloader its self.
                         */
                       if ((Address < FLASH_CODE_AREA_START) || (Address >= FLASH_CODE_AREA_END))
                        {
                           if (Address >= RAM_AREA_START) break;
                            EndFlashProgramming();
                            //f_close(&SrecFile);
                            return (2);
                        }

                        /*
                         * Calculate the actual byte count from the s-record count field
                         * Remove address (3 bytes) & checksum (1 bytes) to get the number of data bytes
                         */
                        ByteCount = (ByteCount - 3);
                        /*
                         * Set pointer to the first character of the data field
                         * start of data field = record type(2) + count(2) + address(6)
                         */
                        p = buffer+8;
                        for (uint32_t i=0; i<ByteCount; i++)
                        {
                           data[i] = (uint8_t)strtol(strncpy(hex, p+(i*2),2), &q, 16);
                        }
                        write_flash(Address, data, ByteCount);
                        break;
            case '2' :  // 24 Bit Address Record
                        strncpy(addr,p,6);
                        Address = (uint32_t)strtol(addr,&p, 16);  // Get the address of this line of data
                        /*
                         * Range check for valid area - we only support address range outside of the
                         * bootloader its self.
                         */
                       if ((Address < FLASH_CODE_AREA_START) || (Address >= FLASH_CODE_AREA_END))
                        {
                           if (Address >= RAM_AREA_START) break;
                            EndFlashProgramming();
                            //f_close(&SrecFile);
                            return (2);
                        }

                        /*
                         * Calculate the actual byte count from the s-record count field
                         * Remove address (3 bytes) & checksum (1 bytes) to get the number of data bytes
                         */
                        ByteCount = (ByteCount - 4);
                        /*
                         * Set pointer to the first character of the data field
                         * start of data field = record type(2) + count(2) + address(6)
                         */
                        p = buffer+10;
                        for (uint32_t i=0; i<ByteCount; i++)
                        {
                           data[i] = (uint8_t)strtol(strncpy(hex, p+(i*2),2), &q, 16);
                        }
                        write_flash(Address, data, ByteCount);
                        break;
            case '3' :  // 32 Bit Address Record
                        APP_PRINT("32 bit addr rec\r\n");
                        strncpy(addr,p,8);
                        Address = (uint32_t)strtol(addr,&p, 16);  // Get the address of this line of data
                        /*
                         * Range check for valid area - we only support address range outside of the
                         * bootloader its self.
                         */
                        if ((Address < FLASH_CODE_AREA_START) || (Address >= FLASH_CODE_AREA_END))
                        {
                            if ((Address >= RAM_AREA_START)||(Address == OSIS_CONFIG_AREA)) break;
                            EndFlashProgramming();
                            //f_close(&SrecFile);
                            return (2);
                        }
                        /*
                         * Calculate the actual byte count from the s-record count field
                         * Remove address (4 bytes) & checksum (1 bytes) to get the number of data bytes
                         */
                        ByteCount = (ByteCount - 5);
                        /*
                         * Set pointer to the first character of the data field
                         * start of data field = record type(2) + count(2) + address(8)
                         */
                        p = buffer+12;
                        for (uint32_t i=0; i<ByteCount; i++)
                        {
                           data[i] = (uint8_t)strtol(strncpy(hex, p+(i*2),2), &q, 16);
                        }
                        write_flash(Address, data, ByteCount);
                        break;
            case '5' :  // Ignore the count record(s)
            case '6' :
                        break;
            case '7' :  // Start Address Record (Termination of the s-rec file)
            case '8' :
            case '9' :
                        EndFlashProgramming();
                        break;
            default :
                //f_close(&SrecFile);
                return (1);
        }
    } while (update_timerout);

    //f_close(&SrecFile);
    return(0);
}

