#include "hal_data.h"
#include "common_utils.h"
#include "rtc.h"
#include "data_flash_helper.h"

#define APP_START_ADDR                  0x20000

typedef int (*main_fnptr)(void); // Prototype for function pointer to the image

FSP_CPP_HEADER
void R_BSP_WarmStart(bsp_warm_start_event_t event);
FSP_CPP_FOOTER

static void execute_user_code(uint32_t address);
static void reset_all_leds_states(void);
static void LedDisplay(uint8_t tmp);

session_header_t                    device_setup_session;

uint32_t                     image_file_size;
/*******************************************************************************************************************//**
 * main() is generated by the RA Configuration editor and is used to generate threads if an RTOS is used.  This function
 * is called by main() when no RTOS is used.
 **********************************************************************************************************************/
void hal_entry(void)
{
    /* TODO: add your own code here */
    fsp_err_t  error = FSP_SUCCESS;
    bool       update_ok = true;
    bool       app_jump_execution = false;

    reset_all_leds_states();

    /* Initialize System timer on RTC */
    error = rtc_init();
    if (FSP_SUCCESS != error)
    {
        /* RTC module init failed */
        APP_ERR_PRINT("\r\n ** RTC INIT FAILED ** \r\n");
        LedDisplay(3);
        APP_ERR_TRAP(error);
    }

 //   APP_PRINT ("Initiate System Calendar Date and Time\r\n");
    error = set_rtc_calendar_time();
    if (FSP_SUCCESS != error)
    {
        rtc_deinit();
        APP_ERR_PRINT("\r\nRTC Calendar Time Set failed.\r\nClosing the driver. Restart the Application\r\n");
        LedDisplay(3);
        APP_ERR_TRAP(error);
    }
    else
    {
        APP_PRINT("\r\nSystem Time is set successfully\r\n");
    }

    /* Initialize flash driver */
    error = internal_storage_init();
    if (FSP_SUCCESS != error)
    {
        APP_ERR_PRINT("\r\n ** INTERNAL FLASH INIT FAILED ** \r\n");
        LedDisplay(3);
        APP_ERR_TRAP(error);
    }

    /* Read the persistent device setup log from DATA FLASH */
    memset (&device_setup_session, 0, sizeof(device_setup_session));
    error = internal_storage_read ((uint8_t *) &device_setup_session, sizeof(session_header_t), PERSISTENT_LOG_METADATA_TYPE);
    if(FSP_SUCCESS != error)
    {
        LedDisplay(3);
        APP_PRINT("\r\nPersistent device setup log is not initialized yet\r\n");
    }

    /* Open USB instance */
    error = R_USB_Open (&g_basic0_ctrl, &g_basic0_cfg);
    /* Handle error */
    if (FSP_SUCCESS != error)
    {
        /* Turn ON RED LED to indicate fatal error */
        //TURN_RED_ON
        APP_ERR_TRAP(error);
    }

//    /* Initialize USB HMSC */
//    error = usb_init();
//    if (FSP_SUCCESS != error)
//    {
//        APP_ERR_PRINT ("\r\nError in initializing FAT on USB_HMSC\r\n");
//        LedDisplay(3);
//    }
//    else
//    {
//        if(firmware_image_exists("0:Put Image Here/update.txt"))
//        {
//            update_ok = update_device_app();
//            if(!update_ok)
//            {
//                LedDisplay(3);
//            }
//        }
//
//        if(firmware_image_exists("0:Put Image Here/loaded.txt") && update_ok)
//        {
//            app_jump_execution = true;
//            LedDisplay(2);
//            APP_PRINT ("\r\nComplete firmware update, and execute Safely Eject\r\n");
//            usb_safely_eject ();
//        }
//
//        /* erase persistent log */
//        error = internal_storage_erase(PERSISTENT_LOG_METADATA_TYPE);
//        if (FSP_SUCCESS != error)
//        {
//            LedDisplay(3);
//            APP_ERR_TRAP(error);
//        }
//
//        device_setup_session.fw_image_size = image_file_size;
//        device_setup_session.is_fw_loaded = 1;
//        device_setup_session.valid = 0xAA55;
//        if (FSP_SUCCESS != internal_storage_write ((uint8_t *) &device_setup_session, sizeof(session_header_t), PERSISTENT_LOG_METADATA_TYPE))
//        {
//            APP_ERR_PRINT ("\r\nWrite Data Flash Fail\r\n");
//            LedDisplay(3);
//            APP_ERR_TRAP(error);
//        }
//    }

    if(USBParseSrecFile() == 0)
    {
        /* erase persistent log */
        error = internal_storage_erase(PERSISTENT_LOG_METADATA_TYPE);
        if (FSP_SUCCESS != error)
        {
            LedDisplay(3);
            APP_ERR_TRAP(error);
        }

        device_setup_session.fw_image_size = image_file_size;
        device_setup_session.is_fw_loaded = 1;
        device_setup_session.valid = 0xAA55;
        if (FSP_SUCCESS != internal_storage_write ((uint8_t *) &device_setup_session, sizeof(session_header_t), PERSISTENT_LOG_METADATA_TYPE))
        {
            APP_ERR_PRINT ("\r\nWrite Data Flash Fail\r\n");
            LedDisplay(3);
            APP_ERR_TRAP(error);
        }
    }

    internal_storage_deinit();

    // If the app has been loaded and
    if((app_jump_execution) || ((device_setup_session.valid == 0xAA55) && (device_setup_session.is_fw_loaded == 1)))
    {
        APP_PRINT ("\r\nExecuting JUMP\r\n");
        execute_user_code(APP_START_ADDR);
    }

    while (true)
    {
        LedDisplay(0);
        R_BSP_SoftwareDelay(1, BSP_DELAY_UNITS_SECONDS);

        LedDisplay(4);
        R_BSP_SoftwareDelay(1, BSP_DELAY_UNITS_SECONDS);
    }
#if BSP_TZ_SECURE_BUILD
    /* Enter non-secure code */
    R_BSP_NonSecureEnter();
#endif
}

/*******************************************************************************************************************//**
 * This function is called at various points during the startup process.  This implementation uses the event that is
 * called right before main() to set up the pins.
 *
 * @param[in]  event    Where at in the start up process the code is currently at
 **********************************************************************************************************************/
void R_BSP_WarmStart(bsp_warm_start_event_t event)
{
    if (BSP_WARM_START_RESET == event)
    {
#if BSP_FEATURE_FLASH_LP_VERSION != 0

        /* Enable reading from data flash. */
        R_FACI_LP->DFLCTL = 1U;

        /* Would normally have to wait tDSTOP(6us) for data flash recovery. Placing the enable here, before clock and
         * C runtime initialization, should negate the need for a delay since the initialization will typically take more than 6us. */
#endif
    }

    if (BSP_WARM_START_POST_C == event)
    {
        /* C runtime environment and system clocks are setup. */

        /* Configure pins. */
        R_IOPORT_Open (&g_ioport_ctrl, &g_bsp_pin_cfg);
    }
}

#if BSP_TZ_SECURE_BUILD

BSP_CMSE_NONSECURE_ENTRY void template_nonsecure_callable ();

/* Trustzone Secure Projects require at least one nonsecure callable function in order to build (Remove this if it is not required to build). */
BSP_CMSE_NONSECURE_ENTRY void template_nonsecure_callable ()
{

}
#endif

static void execute_user_code(uint32_t address)
{
    main_fnptr *p_jump_to_app; // Function pointer main that will be used to jump to application

    __disable_irq();

#if 0       //Commented out 2022.06.24 WangJin
    /* Disable the stack monitor */
    R_MPU_SPMON->SP[0].PT = 0xA500U;                /* Enable access to Main Stack Pointer Monitor Access Control Register */
    R_MPU_SPMON->SP[1].PT = 0xA500U;                /* Enable access to Process Stack Pointer Monitor Access Control Register */
    R_MPU_SPMON->SP[0].CTL = 0U;                    /* disable the Main stack monitor */
    R_MPU_SPMON->SP[1].CTL = 0U;                    /* disable the Process stack monitor */
#else
    __set_MSPLIM(0);
#endif
    SysTick->CTRL  = 0;                             /* Disable the systick timer */
    NVIC_DisableIRQ ( SysTick_IRQn );               /* Disable the systick timer IRQ */
    NVIC_ClearPendingIRQ( SysTick_IRQn );           /* Clear any pending systick timer IRQ */

    R_USB_FS0->SYSCFG = 0;                            /* Disable USB FS */
    R_USB_FS0->DVSTCTR0 = 0;
    R_USB_FS0->INTENB0 = 0;
    R_USB_FS0->INTENB1 = 0;
    R_USB_FS0->INTSTS0 = 0;
    R_USB_FS0->INTSTS1 = 0;
    R_USB_FS0->BRDYENB = 0;
    R_USB_FS0->NRDYENB = 0;
    R_USB_FS0->BEMPENB =0;
    R_USB_FS0->USBADDR = 0;

    // Clear all interrupts!
    // ICU Event Link Setting Register i (IELSRi) (RA6M3: i = 0 to 95)
    for (uint32_t i = 0; i<96; i++)
    {
        R_ICU->IELSR[i] = 0;
    }

    for (uint32_t i = 0; i<8; i++)
    {
        NVIC->ICER[i] = 0xFFFFFFFF;
        NVIC->ICPR[i] = 0xFFFFFFFF;
    }

    // IRQ Control Register i (IRQCRi) (RA6M3: i = 0 to 15)
    for  (uint32_t i = 0; i<16; i++)
    {
        R_ICU->IRQCR[i] = 0;
    }

    p_jump_to_app = (main_fnptr*)(address+4);

    SCB->VTOR = (address & 0x1FFFFF80);

    /* Complete all explicit memory accesses */
    __DSB();
    /* Flush the pipeline in the processor */
    __ISB();

    // Flush the ROM cache, or we won't see the changed data
    R_FCACHE->FCACHEE_b.FCACHEEN = 0;           //Disable Flash Memory Cache
    R_FCACHE->FCACHEIV_b.FCACHEIV = 1;          //Invalidate Flash Cache
    while (0 != R_FCACHE->FCACHEIV_b.FCACHEIV);
    R_FCACHE->FCACHEE_b.FCACHEEN = 1;           // Re-Enable ROM cache

    /* Set bits 20-23 to enable CP10 and CP11 coprocessor */
    SCB->CPACR = 0;

     /** Set stack here. */
    __set_MSP(*((uint32_t*)(address)));

    /* Ensure that when we jump we're using the Main Stack Pointer */
    __set_CONTROL(0);

    /* Clear lazy stacking for FPU operations */
    FPU->FPCCR &= 0xC0000000;

    /** Jump to image*/
     (*p_jump_to_app)();
     __NOP();
}

static void reset_all_leds_states(void)
{
    R_IOPORT_PinWrite(&g_ioport_ctrl, LED1, LED_OFF);
    R_IOPORT_PinWrite(&g_ioport_ctrl, LED2, LED_OFF);
    R_IOPORT_PinWrite(&g_ioport_ctrl, LED3, LED_OFF);
}

static void LedDisplay(uint8_t tmp)
{
    R_IOPORT_PinWrite(&g_ioport_ctrl, LED1, LED_OFF);
    R_IOPORT_PinWrite(&g_ioport_ctrl, LED2, LED_OFF);
    R_IOPORT_PinWrite(&g_ioport_ctrl, LED3, LED_OFF);

    switch (tmp)
    {
        case 1:
            R_IOPORT_PinWrite(&g_ioport_ctrl, LED1, LED_ON);
            break;
        case 2:
            R_IOPORT_PinWrite(&g_ioport_ctrl, LED2, LED_ON);
            break;
        case 3:
            R_IOPORT_PinWrite(&g_ioport_ctrl, LED3, LED_ON);
            break;
        case 4:
            R_IOPORT_PinWrite(&g_ioport_ctrl, LED1, LED_ON);
            R_IOPORT_PinWrite(&g_ioport_ctrl, LED2, LED_ON);
            R_IOPORT_PinWrite(&g_ioport_ctrl, LED3, LED_ON);
            break;
        default:
            break;
    }
}
